from django.db import models
from django.contrib.auth.models import User, Group
from mptt.models import MPTTModel

class Menu(MPTTModel):
    """MPTT tree menu"""
    name = models.CharField(max_length=20)
    url = models.CharField(max_length=100, db_index=True)
    user = models.ForeignKey(User)
    group = models.ForeignKey(Group, null=True, blank=True)
    updated_date = models.DateTimeField(auto_now=True)
    parent = models.ForeignKey('self', null=True, blank=True, related_name='children')

    class Meta:
        ordering = ["name"]
        verbose_name, verbose_name_plural = "menu", "menus"
        db_table = 'samklang_menu'

    class MPTTMeta:
        pass

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('Menu', [self.id])




from django.conf.urls.defaults import *
from samklang_menu.views import list_menu, new_menuitem, delete_menuitem

urlpatterns = patterns('',
    (r'^list/(?P<tree_id>\d+)$', list_menu, {}, 'strekmann_menu_list'),
    (r'^new/(?P<tree_id>\d+)$', new_menuitem, {}, 'strekmann_menu_new'),
    (r'^delete/(?P<tree_id>\d+)$', delete_menuitem, {}, 'strekmann_menu_delete'),
)



from samklang_menu.models import Menu
from django import template

register = template.Library()

def make_menu(parent, user):
    menu = []
    for child in parent.get_children():
        if not child.group or child.group in user.groups.all():
            menu.append(child)
    return menu

def html_menu(menu):
    return "<ul>" + "".join([ '<li><a href="%s">%s</a></li>' % (item.url, item.name) for item in menu ]) + "</ul>"

@register.simple_tag
def menu(request, default_url="/"):
    if hasattr(request, 'site'):
        site_id = request.site.id
    else:
        site_id = 1
    user = request.user
    path = request.path_info
    try:
        active_link = Menu.objects.get(tree_id=site_id, url=path)
        menu = make_menu(active_link, user)
        while not menu and active_link.parent: #empty?
            active_link = active_link.parent
            menu = make_menu(active_link, user)
    except Menu.DoesNotExist:
        active_link = Menu.objects.get(tree_id=site_id, url=default_url)
        menu = make_menu(active_link, user)
        while not menu and active_link.parent:
            active_link = active_link.parent
            menu = make_menu(active_link, user)

    return html_menu(menu)

@register.simple_tag
def breadcrumbs(request, extra=[]):
    """Get the menu item for the page you are at. Find all ancestors in the menu hierarchy. Start at top level and break if permissions are not ok.

    You can pass extra elements using the "extra" parameter"""

    if hasattr(request, 'site'):
        site_id = request.site.id
    else:
        site_id = 1
    user = request.user
    path = request.path_info
    try:
        link = Menu.objects.get(tree_id=site_id, url=path)
    except Menu.DoesNotExist:
        link = Menu.objects.get(tree_id=site_id, url='/')

    ancestors = []
    for ancestor in link.get_ancestors():
        if not ancestor.group or ancestor.group in user.groups.all():
            ancestors.append(ancestor)
        else:
            break
    ancestors.append(link)

    return " > ".join([ '<a href="%s">%s</a>' % (ancestor.url, ancestor.name) for ancestor in ancestors ] + [ '<a href="%s">%s</a>' % (t[0], t[1]) for t in extra ])

